import { z } from "zod";
import { EmptyZodObject, NextJSRequestHandler, RouteParamsBase } from "./handler-types";
import { Expand, ExtendZodObject, MergeZodObjects } from "./type-helpers";
import { UseFinishRequest } from "./use-finish-request";
export declare class BaseRequestHandler<RouteParams extends RouteParamsBase = object, QueryParams extends z.SomeZodObject = EmptyZodObject, Cookies extends z.SomeZodObject = EmptyZodObject, Body extends z.SomeZodObject = EmptyZodObject> extends UseFinishRequest<RouteParams, QueryParams, Cookies, Body, object, object> {
    private queryParamsSchema;
    private bodySchema;
    private cookieSchema;
    getBaseHandler(): this;
    getContextFunction(): () => {};
    constructor(queryParamsSchema: QueryParams, cookieSchema: Cookies, bodySchema: Body);
    params<RP extends string>(..._params: readonly RP[]): BaseRequestHandler<Expand<{
        [K in RP]: string;
    } & RouteParams>, QueryParams, Cookies, Body>;
    catchAllParams<RP extends string>(_segment: RP): BaseRequestHandler<Expand<{
        [K in RP]: string[];
    } & RouteParams>, QueryParams, Cookies, Body>;
    query<O extends z.ZodRawShape>(schema: O): BaseRequestHandler<RouteParams, ExtendZodObject<QueryParams, O>, Cookies, Body>;
    cookies<O extends z.ZodRawShape>(schema: O): BaseRequestHandler<RouteParams, QueryParams, ExtendZodObject<Cookies, O>, Body>;
    body<O extends z.ZodRawShape>(schema: O): BaseRequestHandler<RouteParams, QueryParams, Cookies, ExtendZodObject<Body, O>>;
    extend<R extends RouteParamsBase, Q extends z.AnyZodObject, C extends z.AnyZodObject, B extends z.AnyZodObject>(other: BaseRequestHandler<R, Q, C, B>): BaseRequestHandler<RouteParams & R, MergeZodObjects<QueryParams, Q>, MergeZodObjects<Cookies, C>, MergeZodObjects<Body, B>>;
    parseRequest(...[req, { params }]: Parameters<NextJSRequestHandler<RouteParams>>): {
        query: z.infer<QueryParams>;
        body: z.infer<Body>;
        cookies: z.infer<Cookies>;
        params: RouteParams;
    };
    getRequirements(): this;
}
