"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "codegen", {
    enumerable: true,
    get: function() {
        return codegen;
    }
});
var _typedapi = require("./typed-api");
function _array_like_to_array(arr, len) {
    if (len == null || len > arr.length) len = arr.length;
    for(var i = 0, arr2 = new Array(len); i < len; i++)arr2[i] = arr[i];
    return arr2;
}
function _array_with_holes(arr) {
    if (Array.isArray(arr)) return arr;
}
function _iterable_to_array_limit(arr, i) {
    var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"];
    if (_i == null) return;
    var _arr = [];
    var _n = true;
    var _d = false;
    var _s, _e;
    try {
        for(_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true){
            _arr.push(_s.value);
            if (i && _arr.length === i) break;
        }
    } catch (err) {
        _d = true;
        _e = err;
    } finally{
        try {
            if (!_n && _i["return"] != null) _i["return"]();
        } finally{
            if (_d) throw _e;
        }
    }
    return _arr;
}
function _non_iterable_rest() {
    throw new TypeError("Invalid attempt to destructure non-iterable instance.\\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}
function _sliced_to_array(arr, i) {
    return _array_with_holes(arr) || _iterable_to_array_limit(arr, i) || _unsupported_iterable_to_array(arr, i) || _non_iterable_rest();
}
function _unsupported_iterable_to_array(o, minLen) {
    if (!o) return;
    if (typeof o === "string") return _array_like_to_array(o, minLen);
    var n = Object.prototype.toString.call(o).slice(8, -1);
    if (n === "Object" && o.constructor) n = o.constructor.name;
    if (n === "Map" || n === "Set") return Array.from(n);
    if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _array_like_to_array(o, minLen);
}
function codegen(paths, baseFolderPath) {
    var imports = [];
    var params = [];
    var asserts = [];
    var routes = {
        DELETE: [],
        GET: [],
        HEAD: [],
        OPTIONS: [],
        PATCH: [],
        POST: [],
        PUT: []
    };
    var currentIndex = 0;
    var _iteratorNormalCompletion = true, _didIteratorError = false, _iteratorError = undefined;
    try {
        var _loop = function() {
            var path = _step.value;
            var importId = "Route".concat(currentIndex++);
            var importPath = "./" + path.filepath.slice(baseFolderPath.length + 1, -3);
            var methods = Object.entries(path.methods).filter(function(param) {
                var _param = _sliced_to_array(param, 2), hasMethod = _param[1];
                return hasMethod;
            }).map(function(param) {
                var _param = _sliced_to_array(param, 1), method = _param[0];
                return method;
            });
            imports.push("import type { ".concat(methods.map(function(m) {
                return "".concat(m, " as ").concat(importId).concat(m);
            }).join(", "), ' } from "').concat(importPath, '";'));
            var paramsTypeArgs = Object.entries(path.params).map(function(param) {
                var _param = _sliced_to_array(param, 2), _$param = _param[0], type = _param[1];
                return "".concat(_$param, ":").concat(type === "string" ? "string" : "string[]", ";");
            }).join("");
            if (paramsTypeArgs.length > 0) params.push("type ".concat(importId, "Params = {").concat(paramsTypeArgs, "};"));
            else params.push("type ".concat(importId, "Params = Record<string, never>;"));
            methods.forEach(function(m) {
                asserts.push("Assert<APIType<typeof ".concat(importId).concat(m, '>["method"], "').concat(m, '">'));
                asserts.push("Assert<APIType<typeof ".concat(importId).concat(m, '>["routeParams"], ').concat(importId, "Params>"));
                routes[m].push("'".concat(path.url, "': { api: APIType<typeof ").concat(importId).concat(m, ">; params: ").concat(importId, "Params; };"));
            });
        };
        for(var _iterator = paths[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true)_loop();
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally{
        try {
            if (!_iteratorNormalCompletion && _iterator.return != null) {
                _iterator.return();
            }
        } finally{
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }
    var routeDict = Object.entries(routes).filter(function(param) {
        var _param = _sliced_to_array(param, 2), r = _param[1];
        return r.length;
    }).map(function(param) {
        var _param = _sliced_to_array(param, 2), method = _param[0], routes = _param[1];
        return "type Routes".concat(method, " = {\n    ").concat(routes.join("\n    "), "\n};");
    });
    var functions = Object.entries(routes).filter(function(param) {
        var _param = _sliced_to_array(param, 2), r = _param[1];
        return r.length;
    }).map(function(param) {
        var _param = _sliced_to_array(param, 1), method = _param[0];
        return "export async function api".concat(method, "<Route extends keyof Routes").concat(method, ", ThrowErrors extends boolean = true>(\n    route: Route,\n    data: RemoveNever<{\n        throwErrors?: ThrowErrors,\n        options?: RequestInit,\n        params: EmptyRecordToNever<Routes").concat(method, '[Route]["params"]>,\n        query: ObjectToNever<Routes').concat(method, '[Route]["api"]["queryParams"]>,').concat(_typedapi.RequestMethodHasBody[method] ? "\n        body: ObjectToNever<Routes".concat(method, '[Route]["api"]["body"]>,') : "", "\n    }>,\n): Promise<WithErrors<Routes").concat(method, '[Route]["api"]["data"], RequestError, ThrowErrors>> {\n    const dataAny = data as any;\n    const shouldThrowErrors = dataAny.throwErrors ?? true;\n\n    try {\n        \n        const query = new URLSearchParams(dataAny.query as any);\n        const queryStr = query.toString().length > 0\n            ? "?" + query.toString()\n            : "";\n        const response = await fetch(buildUrl(route, dataAny.params) + queryStr, {\n            method: "').concat(method, '",\n            headers: {\n            "Content-Type": "application/json",\n            "Accept": "application/json",\n            },\n            ').concat(_typedapi.RequestMethodHasBody[method] ? "body: JSON.stringify(dataAny.body)," : "", "\n            ...dataAny.options,\n        });\n\n        if (!response.ok) {\n            const errorType = errorTypeFromCode(response.status);\n            const message = await response.text();\n            const error = new RequestError(errorType, message);\n            throw error;\n        }\n\n        const responseData = await response.json() as APIResponseWrapper<Routes").concat(method, '[Route]["api"]["data"]>;\n\n        if (responseData.status === "error") {\n            const error = new RequestError("server-error", responseData.message);\n            throw error;\n        }\n\n        return responseData.data as any;\n    } catch (e) {\n        const error = e instanceof RequestError ? e : new RequestError("network-error", (e as Error).message);\n        if (shouldThrowErrors) throw error;\n        return {status: "error", error: error} as any;\n    }\n}');
    });
    return 'import type { APIType, APIResponseWrapper } from "next-typed-api/lib"\n  \n'.concat(imports.join("\n"), "\n\n").concat(params.join("\n"), "\n\n").concat(routeDict.join("\n"), "\n\n").concat(functions.join("\n"), "\n\ntype _Asserts = [\n    ").concat(asserts.join(",\n    "), '\n];\n\ntype Assert<T, U extends T> = T;\ntype WithErrors<T, E, ThrowErrors extends boolean | undefined> = ThrowErrors extends false ? {status: "error", error: E} | {"status": "ok", data: T} : T;\ntype RemoveNever<T> = Pick<\n  T,\n  {\n    [K in keyof T]: T[K] extends never ? never : K;\n  }[keyof T]\n>;\ntype ObjectToNever<T> = object extends T ? never : T;\ntype EmptyRecordToNever<T> = Record<string, never> extends T ? never : T;\n\nexport type RequestErrorType = "unauthorized" | "bad-request" | "not-found" | "server-error" | "forbidden" | "network-error" | "unknown";\n\nexport class RequestError extends Error {\n    public readonly type: RequestErrorType;\n    constructor(type: RequestErrorType, message: string) {\n        super(message);\n        this.type = type;\n    }\n}\n\nfunction errorTypeFromCode(code: number): RequestErrorType {\n    if (code === 401) return "unauthorized";\n    if (code === 403) return "forbidden";\n    if (code === 404) return "not-found";\n    if (code >= 500) return "server-error";\n    if (code >= 400) return "bad-request";\n    return "unknown";\n}\n\nfunction buildUrl(path: string, params: Record<string, string | string[]>) {\n    const parts = path.split("/");\n    const url = parts\n      .map((part) => {\n        if (part.startsWith(":")) {\n          const param = part.slice(1);\n          const value = params[param];\n          return value as string;\n        } else if (part.startsWith("*")) {\n          const param = part.slice(1);\n          const value = params[param];\n          return (value as string[]).join("/");\n        } else return part;\n      })\n      .join("/");\n    return url;\n}\n');
}
